<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

require_once 'config.php';
require_once 'queries.php';
require_once 'lib/helper_functions.php';
require_once 'MDB2.php';
require_once 'Log.php';
require_once 'vtsubmit.php';

#Set header for authentication.
if (!isset($_SERVER['PHP_AUTH_USER'])) {
	header('WWW-Authenticate: Basic realm="My Realm"');
	header('HTTP/1.0 401 Unauthorized');
} 
else 
{

# Initialize logging
$logger = &Log::singleton('file', "$mainLogFile", 'NEPENTHESFE_CRON', array(), constant($mainLogLevel));

if (PEAR::isError($logger)) {
	die("Can not create logfile");
}

# Initialize database
$db =& MDB2::connect($mainDbdsn, $mainDatabaseOptions);

if (PEAR::isError($db)) {
	$logger->log("Failed to connect to database:", PEAR_LOG_PEAR_LOG_EMERG);
	$logger->log($db->getMessage, PEAR_LOG_EMERG);
    exit;
}

# Authenticate sensor with HTTP Basic Auth via DB
$query = sprintf($queries['auth_sensor'], $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], $_SERVER['REMOTE_ADDR']);

$res = $db->query($query);

# A serious DB error occured - log and bail out
if (PEAR::isError($res)) {
        $logger->log("DB error during client authentication:", PEAR_LOG_EMERG);
        $logger->log($res->getMessage(), PEAR_LOG_EMERG);
        exit;
}

# Authentication failed for a sensor
if ($res->numRows() == 0) {
        $logger->log("Authentication failed for client $_SERVER[REMOTE_ADDR] with username $_SERVER[PHP_AUTH_USER]", 
                PEAR_LOG_WARNING);
//		exit;
        
}

$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
$sensor_idx = $row['sensor_idx'];

$query = sprintf($queries['update_sensor_last_submission'], time(), $sensor_idx);
$db->query($query);

# Further processing of sample data with modules
foreach($mainModules as $module_name => $module_options) {
                
	if ($module_options['enable'] && ereg('cron', $module_options['scope'])
		&& $module_options['type'] == 'sample') {
	
		$logger->log("Retrieving sample_info from module $module_name", PEAR_LOG_INFO);
					
		require_once("modules/$module_name/$module_name.php");
	                	
		$module = new $module_name();
	                	 
		$module_param = $module->param;

		if (!$sample_info_details = $module->cron()) {
			break;
		}

		for ($i=0; $i<count($sample_info_details); $i++) {	

			# Check if this particular info already exists in DB
			$query = sprintf($queries['select_sample_by_md5'], 
				$sample_info_details[$i]['md5']);

			$logger->log("Query: $query", PEAR_LOG_DEBUG);
				
			$res = $db->query($query);
			
			if (PEAR::isError($res)) {
				$logger->log("DB error:", PEAR_LOG_EMERG); 
				$logger->log($res->getMessage(), PEAR_LOG_EMERG);
				exit;
        	}
        	
			$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

			$sample_info_details[$i]['sha512'] = $row['sample_sha512'];
			
			$query = sprintf($queries['select_exists_sample_info'],
				$module->info_type,
				$module->info_name,
				$sample_info_details[$i]['sha512']);
			
			$logger->log("Query: $query", PEAR_LOG_DEBUG);
			
			$res = $db->query($query);

		    if (PEAR::isError($res)) {
				$logger->log("DB error:", PEAR_LOG_EMERG); 
				$logger->log($res->getMessage(), PEAR_LOG_EMERG);
				exit;
        	}
        
			$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

			if (PEAR::isError($row)) {
				$logger->log("DB error:", PEAR_LOG_EMERG); 
				$logger->log($row->getMessage(), PEAR_LOG_EMERG);
				exit;
        	}
        	
			if ($res->numRows() == 0 && !empty($sample_info_details[$i]['sha512'])) {
				$query = sprintf($queries['insert_sample_info'], 
					$db->nextID('sample_info'),
					$module->info_type,
					$module->info_name,	
					$sample_info_details[$i]['sha512'],
					$sample_info_details[$i]['result_xml']
				);

				$logger->log("Query: $query", PEAR_LOG_DEBUG);
				
				$db->query($query);
			}
		}
	}
}

# Update RRDs
foreach ($mainStats as $stat_key => $stat_options) {
	$logger->log("Running RRD update for $stat_key", PEAR_LOG_DEBUG);
	
	if ($stat_options['enable'] != TRUE) {
		break;
	}

	$query = sprintf($queries['select_stat_by_key'], $stat_key);
	$logger->log("RRD update query: $query", PEAR_LOG_DEBUG);
	
	$res = $db->query($query);

	while ($row = $res->fetchRow(MDB2_FETCHMODE_ASSOC)) {
		updateRRD($stat_key, $row['stat_value'], $row['stat_counter']);
	}
}

}
?>
