<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

error_reporting(0);
ob_start();

require_once 'config.php';
require_once 'queries.php';
require_once 'lib/helper_functions.php';
require_once 'MDB2.php';
require_once 'HTML/Template/IT.php';
require_once 'Pager/Pager.php';
require_once 'Pager/Pager_Wrapper.php';
require_once 'Auth.php';
require_once 'Log.php';

# Initialize logging
$logger = &Log::singleton('file', "$mainLogFile", 'NEPENTHESFE', array(), constant($mainLogLevel));

if (PEAR::isError($logger)) {
	die("Can not create logfile");
}

# Set language/locale parameters
$logger->log("Setting language parameters", PEAR_LOG_DEBUG);
if (!isset($_GET["lang"]) || empty($_GET["lang"])) {
	$language = $mainLanguages[$mainLang];
}
else {
	$language = $mainLanguages[$_GET["lang"]];
}

putenv("LANG=$language"); 
setlocale(LC_ALL, $language);
bindtextdomain($mainGettextDomain, $mainLangPath);
textdomain($mainGettextDomain);

header("Cache-Control: no-cache, must-revalidate");
header("Pragma: no-cache");

# Start authentication
$auth = new Auth("MDB2", $mainOptionsAuth, "loginFunction");

$auth->start();

if ($auth->checkAuth()) {
	$db =& MDB2::connect($mainDbdsn, $mainDatabaseOptions);

	if (PEAR::isError($db)) {
		$logger->log("Failed to connect to database:", PEAR_LOG_EMERG);
		$logger->log($db->getMessage, PEAR_LOG_EMERG);
	    exit;
	}
	
	# Update Last Login for User
	$query = sprintf($queries['update_user_last_login'], $auth->session['timestamp'], $auth->session['username']);
	
	$db->query($query);	
	
	# Set default entry page/func
	if (!empty ($_GET["func"])) {
	    $func = $_GET["func"];
	} elseif (!empty($_POST["func"])) {
	    $func = $_POST["func"];
	} else {
		$func = "listinstances";
	}
	
	switch ($func) {
	
	 /* Main Function: listinstances */
	
	 case "listinstances":
	 
		printHeader();
	 	
		# Set default values for ordering and sorting of query
		if (!isset($_GET['oby']) || empty($_GET['oby'])) {
			$oby = 'instance_date';
		} 
		else {
			$oby = $db->escape($_GET['oby']);
		}
		
		if (isset($_GET['sort']) && $_GET['sort'] != 'desc') {
			$sort = 'asc';
		}
		else {
			$sort = 'desc';
		}
		
		$query = $queries['select_count_instances'];
		$num_db_entries = $db->queryOne($query);
		
		$pagerParameters = array(
			'oby' => $oby,
			'sort' => $sort
		);
	
		$pagerOptions = array(
			'mode'    => 'Sliding',
	        'perPage' => $mainHashesPerPage,
			'delta' => 1,
			'totalItems' => $num_db_entries,
		);
		
		$query = sprintf($queries['select_instances'], $oby, $sort);
		$paged_data = Pager_Wrapper_MDB2($db, $query, $pagerOptions);	
	
		$pager1 = & Pager::factory($pagerParameters);
	
		$pager = & Pager::factory($pagerOptions);
		
		$links = $pager->getLinks();
			
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("listinstances.php", false, true); 
	
		$t->setVariable("PAGER_LINKS", $pager->links);
		$t->setVariable("URL_LISTHITS_BY_SOURCE_UP", "index.php?func=listinstances&oby=instance_source_host&sort=asc&lang=$language");
		$t->setVariable("URL_LISTHITS_BY_SOURCE_DOWN", "index.php?func=listinstances&oby=instance_source_host&sort=desc&lang=$language");
		$t->setVariable("URL_LISTHITS_BY_TARGET_UP", "index.php?func=listinstances&oby=instance_target_host&sort=asc&lang=$language");
		$t->setVariable("URL_LISTHITS_BY_TARGET_DOWN", "index.php?func=listinstances&oby=instance_target_host&sort=desc&lang=$language");
		$t->setVariable("URL_LISTHITS_BY_DATE_UP", "index.php?func=listinstances&oby=instance_date&sort=asc&lang=$language");
		$t->setVariable("URL_LISTHITS_BY_DATE_DOWN", "index.php?func=listinstances&oby=instance_date&sort=desc&lang=$language");
						
		$t->setVariable("TEXT_HASH", gettext("Hash"));
		$t->setVariable("TEXT_SOURCE_IP", gettext("Source IP"));
		$t->setVariable("TEXT_TARGET_IP", gettext("Target IP"));
		$t->setVariable("TEXT_SEEN_DATE", gettext("Date"));	
		$t->setVariable("TEXT_ATTACKER_COUNTRY", gettext("Country"));
		$t->setVariable("TEXT_ATTACKER_CITY", gettext("City"));

		foreach ($paged_data['data'] as $key => $row) {	
			$t->setCurrentBlock("hashes");
			$xml_data = xml2array($row['instance_info_details']);
			
			$t->setVariable("URL_DETAIL_SAMPLE", "index.php?func=detailssample&sample_sha512=" . $row['sample_sha512'] . "&lang=$language");
			$t->setVariable("URL_DETAIL_INSTANCE", "index.php?func=detailsinstance&instance_idx=" . $row['instance_idx'] . "&lang=$language");
			$t->setVariable("source_host", processIP(long2ip($row["instance_source_host"])));
			$t->setVariable("target_host", processIP(long2ip($row["instance_target_host"])));
			$t->setVariable("date", date($mainDateformat, $row['instance_date']));
			$t->setVariable("hash", $row['sample_md5']);
			
			# Check if city is known
			if (isset($xml_data['geoip']['city']['value']) && !empty($xml_data['geoip']['city']['value'])) {
				$t->setVariable("city", $xml_data['geoip']['city']['value']);
			}
			else {
				$t->setVariable("city", gettext("Unknown"));
			}
				
			# Check if a PNG image exists for this country - if not use empty image
			$server_path = preg_replace('/^(.*)\/[^\/]*/', '$1', $_SERVER['SCRIPT_FILENAME']);
			$flag_image = sprintf("%s.png", strtolower($xml_data['geoip']['country_code']['value']));
			if (!file_exists("$server_path/images-flags/$flag_image")) {
				$xml_data['geoip']['country_code']['value'] = "unknown";
			}
			
			$t->setVariable("country_iso", strtolower($xml_data['geoip']['country_code']['value']));
			$t->setVariable("country_long", $xml_data['geoip']['country_name']['value']);			
			$t->setVariable("URL_VISUAL_COUNTRY", "index.php?func=mkv&base=" . $xml_data['geoip']['country_name']['value'] . "&tag=3&lang=$language");
			$t->parseCurrentBlock("hashes");	
		}
			
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	
	 /* Main Function: detailsinstance */
	
	 case "detailsinstance":
	 
		printHeader();
	 
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("detailsinstance.php", false, true); 	
		
		$query = sprintf($queries["select_instance_details"], $db->escape($_GET['instance_idx']));
		
		$res = $db->query($query);
		
		if (PEAR::isError($res)) {
			die($res->getMessage());
		}	
	
		$row = array();
		
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

		$t->setVariable("URL_DETAIL_MALWARE", "index.php?func=detailssample&sample_sha512=".$row['sample_sha512']."&lang=$language");
		$t->setVariable("SAMPLE_SHA512", $row['sample_sha512']);	
		$t->setVariable("TEXT_SAMPLE_MD5", gettext("Hash MD5"));
		$t->setVariable("SAMPLE_MD5", $row['sample_md5']);	
		$t->setVariable("TEXT_SEEN_DATE", gettext("Date of occurrence"));	
		$t->setVariable("SEEN_DATE", date($mainDateformat, $row['instance_date']));
		$t->setVariable("TEXT_INSTANCE_URL", gettext("URL"));	
		$t->setVariable("INSTANCE_URL", processUrl($row["instance_url"]));	
		$t->setVariable("TEXT_INSTANCE_EMAIL", gettext("E-Mail"));
		$t->setVariable("INSTANCE_EMAIL", $row['instance_email']);	
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Sensor"));
		$t->setVariable("URL_DETAIL_SENSOR", "index.php?func=detailssensor&sensor_idx=".$row['sensor_idx']."&lang=$language");	
		$t->setVariable("SENSOR_NAME", $row['sensor_name']);			
		$t->setVariable("TEXT_SOURCE_HOST", gettext("Source IP Address"));
		$t->setVariable("SOURCE_HOST", processIP(long2ip($row["instance_source_host"])));	
		$t->setVariable("TEXT_TARGET_HOST", gettext("Target IP Address"));
		$t->setVariable("TARGET_HOST", processIP(long2ip($row["instance_target_host"])));	
		$t->setVariable("TEXT_INSTANCE_TRIGGER", gettext("Trigger"));
		$t->setVariable("INSTANCE_TRIGGER", processIP($row['instance_trigger']));	
		$t->setVariable("TEXT_INSTANCE_FILENAME", gettext("Filename"));
		$t->setVariable("INSTANCE_FILENAME", $row['instance_filename']);
		$t->setVariable("TEXT_NEPENTHES_DETAILS", gettext("Details obtained from Nepenthes"));
		$t->setVariable("TEXT_SAMPLE_SHA512", gettext("Hash SHA512"));		
		$t->setVariable("TEXT_INSTANCE_FILETYPE", gettext("Filetype"));
		$t->setVariable("INSTANCE_FILETYPE", $row['instance_filetype']);								
		
		# Render additional info retrieved from modules
        foreach($mainModules as $module_name => $module_options) {
			if ($module_options['enable'] && ereg('display', $module_options['scope'])
				&& $module_options['type'] == 'instance') {

				$t->setCurrentBlock("INSTANCE_INFO");
				
				$logger->log("Retrieving instance_info from module $module_name", PEAR_LOG_INFO);
					
				require_once("modules/$module_name/$module_name.php");
				
				$module = new $module_name();

				$query = sprintf($queries['select_instance_info'],
					$db->escape($_GET['instance_idx']),
					$module->info_name
				);

				$db->query($query);
				
				$res = $db->query($query);
				
				if (PEAR::isError($res)) {
					die($res->getMessage());
				}

				$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
				
				$instance_info_details = $module->render($row['instance_info_details']);
				
				$t->setVariable("INSTANCE_INFO_DETAILS", $instance_info_details);
				$t->parseCurrentBlock();	
 			}
		}			
		
		$t->show();
	
		printFooter();	
		
	 break;
	
	  /* Main Function: listsamples */
	
	 case "listsamples":
	 
		printHeader();
		
		# Set default values for ordering and sorting of query
		if (!isset($_GET['oby']) || empty($_GET['oby'])) {
			$oby = 'sample_last_seen';
		} 
		else {
			$oby = $db->escape($_GET['oby']);
		}
		
		if (isset($_GET['sort']) && $_GET['sort'] != 'desc') {
			$sort = 'asc';
		}
		else {
			$sort = 'desc';
		}

		$query = $queries['select_count_samples'];
		$num_db_entries = $db->queryOne($query);
		
		$query = sprintf($queries['select_samples'], $oby, $sort);
		
		$pagerParameters = array(
			'oby' => $oby,
			'sort' => $sort
		);
		
		$pagerOptions = array(
			'mode'    => 'Sliding',
	        'delta'   => 2,
	        'perPage' => $mainHashesPerPage,
			'urlVar' => 'num',
			'totalItems' => $num_db_entries,		
		);
	
		$paged_data = Pager_Wrapper_MDB2($db, $query, $pagerOptions);	
	
		$pager1 = & Pager::factory($pagerParameters);
		
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("listsamples.php", true, true); 
	
		$t->setVariable("PAGER_LINKS", $paged_data["links"]);
		
		$t->setVariable("URL_LIST_SAMPLE_BY_LAST_SEEN_UP", "index.php?func=listsamples&oby=sample_last_seen&sort=asc&lang=$language");
		$t->setVariable("URL_LIST_SAMPLE_BY_LAST_SEEN_DOWN", "index.php?func=listsamples&oby=sample_last_seen&sort=desc&lang=$language");
		$t->setVariable("URL_LIST_SAMPLE_BY_HITCOUNT_UP", "index.php?func=listsamples&oby=sample_hitcount&sort=asc&lang=$language");
		$t->setVariable("URL_LIST_SAMPLE_BY_HITCOUNT_DOWN", "index.php?func=listsamples&oby=sample_hitcount&sort=desc&lang=$language");
				
		$t->setVariable("TEXT_SAMPLE_SHA512", gettext("Hash SHA512"));
		$t->setVariable("TEXT_SAMPLE_MD5", gettext("Hash MD5"));
		$t->setVariable("TEXT_SAMPLE_LAST_SEEN", gettext("Last Seen"));
		$t->setVariable("TEXT_SAMPLE_HITCOUNT", gettext("Hitcount"));
		$t->setVariable("TEXT_VTCOUNT", gettext("VT Count"));
		
		foreach ($paged_data['data'] as $key => $row) {	
			$t->setCurrentBlock("hashes");
			$t->setVariable("URL_DETAIL_SAMPLE", "index.php?func=detailssample&sample_sha512=" . $row['sample_sha512'] . "&lang=$language");
			$t->setVariable("SAMPLE_SHA512", $row["sample_sha512"]);
			$t->setVariable("SAMPLE_MD5", $row["sample_md5"]);
			$t->setVariable("SAMPLE_LAST_SEEN", date($mainDateformat, $row["sample_last_seen"]));
			$t->setVariable("SAMPLE_HITCOUNT", $row["sample_hitcount"]);
			$t->setVariable("SAMPLE_VT_DETECTION", $row["detection_count"]);
			$t->parseCurrentBlock("hashes");					
		}
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	  
	 /* Main Function: detailssample */
	
	 case "detailssample":
	 
		printHeader();
	 
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("detailssample.php", false, true); 	
	
		# Get and print information obtained by Nepenthes	
		$query = sprintf($queries["select_sample_details"], $db->escape($_GET['sample_sha512']));

		$res = $db->query($query);
	
		if (PEAR::isError($res)) {
			die($res->getMessage());
		}	
	
		$row = array();
		
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

		$t->setVariable("TEXT_AV_RESULTS", gettext("Results from Antivirus Scan"));
		$t->setVariable("TEXT_NEPENTHES_DETAILS", gettext("Details obtained from Nepenthes"));
		$t->setVariable("TEXT_HASH_MD5", gettext("Hash MD5"));
		$t->setVariable("HASH_MD5", $row['sample_md5']);
		$t->setVariable("TEXT_HASH_SHA512", gettext("Hash SHA512"));	
		$t->setVariable("HASH_SHA512", $row['sample_sha512']);
		$t->setVariable("TEXT_LASTSEEN", gettext("Last Seen"));		
		$t->setVariable("LASTSEEN", date($mainDateformat, $row['sample_last_seen']));	
		$t->setVariable("TEXT_HITCOUNT", gettext("Hitcount"));	
		$t->setVariable("HITCOUNT", $row["sample_hitcount"]);	
		$t->setVariable("TEXT_STAT_DETAILS", gettext("Statistics"));	
		$t->setVariable("RRD_IMAGE", "sample_sha512_$row[sample_sha512].png");	
		
		# Render additional info retrieved from modules
        foreach($mainModules as $module_name => $module_options) {
			if ($module_options['enable'] && ereg('display', $module_options['scope'])
				&& $module_options['type'] == 'sample') {

				$logger->log("Retrieving sample_info from module $module_name", PEAR_LOG_INFO);
					
				require_once("modules/$module_name/$module_name.php");
				
				$module = new $module_name();

				$query = sprintf($queries['select_sample_info'],
					$db->escape($_GET['sample_sha512']),
					$module->info_name
				);

				$db->query($query);
				
				$res = $db->query($query);
				
				if (PEAR::isError($res)) {
					die($res->getMessage());
				}

				if ($res->numRows() > 0) {
					$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

					$sample_info_details = $module->render($row['sample_info_details']);
	
					if (!empty($sample_info_details)) {
						$t->setCurrentBlock("SAMPLE_INFO");
						$t->setVariable("SAMPLE_INFO_DETAILS", $sample_info_details);
						$t->parseCurrentBlock();	
					}
				}		
 			}
		}			

		$t->show();
		
		printFooter();	
		
	 break;
	 
	 /* Main Function: listsensors */
	 
	 case "listsensors":
	 
		printHeader();
	 	
		# Set default values for ordering and sorting of query
		if (!isset($_GET['oby']) || empty($_GET['oby'])) {
			$oby = 'sensor_idx';
		} 
		else {
			$oby = $db->escape($_GET['oby']);
		}
		
		if (isset($_GET['sort']) && $_GET['sort'] != 'desc') {
			$sort = 'asc';
		}
		else {
			$sort = 'desc';
		}
		
		$query = $queries['select_count_sensors'];
		$num_db_entries = $db->queryOne($query);
		
		$pagerParameters = array(
			'oby' => $oby,
			'sort' => $sort
		);
	
		$pagerOptions = array(
			'mode'    => 'Sliding',
	        'perPage' => $mainHashesPerPage,
			'delta' => 1,
			'totalItems' => $num_db_entries,
		);
		
		$query = sprintf($queries['select_sensors'], $oby, $sort);
		$paged_data = Pager_Wrapper_MDB2($db, $query, $pagerOptions);	
	
		$pager1 = & Pager::factory($pagerParameters);
	
		$pager = & Pager::factory($pagerOptions);
		
		$links = $pager->getLinks();
			
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("listsensors.php", false, true); 
	
		$t->setVariable("PAGER_LINKS", $pager->links);
		$t->setVariable("URL_LISTSENSORS_BY_LAST_SUBMISSION_UP", "index.php?func=listsensors&oby=sensor_last_submission&sort=asc&lang=$language");
		$t->setVariable("URL_LISTSENSORS_BY_LAST_SUBMISSION_DOWN", "index.php?func=listsensors&oby=sensor_last_submission&sort=desc&lang=$language");
						
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Name"));
		$t->setVariable("TEXT_SENSOR_USER", gettext("Username"));
		$t->setVariable("TEXT_SENSOR_IP", gettext("IP Address"));
		$t->setVariable("TEXT_SENSOR_LAST_SUBMISSION", gettext("Last Submission"));	
		$t->setVariable("TEXT_SENSOR_COMMENT", gettext("Comment"));
		$t->setVariable("URL_ADD_SENSOR", "index.php?func=addsensor&lang=$language");
		foreach ($paged_data['data'] as $key => $row) {	
			$t->setCurrentBlock("sensors");

			$t->setVariable("URL_ADD_SENSOR", "index.php?func=addsensor&lang=$language");
			$t->setVariable("URL_DETAIL_SENSOR", "index.php?func=detailssensor&sensor_idx=$row[sensor_idx]&lang=$language");
			$t->setVariable("URL_DELETE_SENSOR", "index.php?func=deleteconfirmsensor&sensor_idx=$row[sensor_idx]&lang=$language");
			$t->setVariable("URL_EDIT_SENSOR", "index.php?func=editsensor&sensor_idx=$row[sensor_idx]&lang=$language");
			$t->setVariable("sensor_name", $row["sensor_name"]);
			$t->setVariable("sensor_user", $row["sensor_user"]);
			$t->setVariable("sensor_ip", processIP($row["sensor_ip"]));
			$t->setVariable("sensor_last_submission", date($mainDateformat, $row['sensor_last_submission']));
			$t->setVariable("sensor_comment", $row["sensor_comment"]);
			
			$t->parseCurrentBlock("sensors");	
		}
			
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 	
	/* Main Function: detailssensor */
	 
	 case "detailssensor":
	 
		printHeader();

		$query = sprintf($queries['select_sensor'], $db->escape($_GET['sensor_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("detailssensor.php", true, true); 

		$t->setVariable("TEXT_SENSOR_DETAILS", gettext("Details for Sensor"));
		$t->setVariable("URL_DELETE_SENSOR", "index.php?func=deleteconfirmsensor&sensor_idx=$row[sensor_idx]&lang=$language");
		$t->setVariable("URL_EDIT_SENSOR", "index.php?func=editsensor&sensor_idx=$row[sensor_idx]&lang=$language");
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Sensor Name"));
		$t->setVariable("SENSOR_NAME", $row['sensor_name']);
		$t->setVariable("TEXT_SENSOR_USER", gettext("Sensor Username"));	
		$t->setVariable("SENSOR_USER", $row['sensor_user']);
		$t->setVariable("TEXT_SENSOR_IP", gettext("Sensor IP"));	
		$t->setVariable("SENSOR_IP", $row['sensor_ip']);
		$t->setVariable("TEXT_SENSOR_COMMENT", gettext("Comment"));	
		$t->setVariable("SENSOR_COMMENT", $row['sensor_comment']);
		$t->setVariable("TEXT_STAT_DETAILS", gettext("Statistics"));	
		$t->setVariable("RRD_IMAGE", "sensor_$row[sensor_idx].png");	
				
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 	 	
	/* Main Function: editsensor */
	 
	 case "editsensor":
	 
		printHeader();

		$query = sprintf($queries['select_sensor'], $db->escape($_GET['sensor_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("editaddsensor.php", true, true); 

		$t->setVariable("URL_POST", "index.php?func=savesensor&lang=$language");
		$t->setVariable("TEXT_SENSOR_ADDEDIT", gettext("Edit Sensor"));
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Sensor Name"));
		$t->setVariable("VALUE_SENSOR_NAME", $row['sensor_name']);
		$t->setVariable("TEXT_SENSOR_USER", gettext("Sensor Username"));	
		$t->setVariable("VALUE_SENSOR_USER", $row['sensor_user']);
		$t->setVariable("TEXT_SENSOR_IP", gettext("Sensor IP"));	
		$t->setVariable("VALUE_SENSOR_IP", $row['sensor_ip']);
		$t->setVariable("TEXT_SENSOR_COMMENT", gettext("Comment"));	
		$t->setVariable("VALUE_SENSOR_COMMENT", $row['sensor_comment']);
		$t->setVariable("TEXT_SENSOR_PASSWORD", gettext("Sensor User Password"));	
		$t->setVariable("VALUE_SENSOR_PASSWORD", $row['sensor_password']);
		$t->setVariable("SENSOR_IDX", $row['sensor_idx']);
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;

	/* Main Function: addsensor */
	 
	 case "addsensor":
	 
		printHeader();
		
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("editaddsensor.php", true, true); 

		$t->setVariable("TEXT_SENSOR_ADDEDIT", gettext("Add Sensor"));
		$t->setVariable("URL_POST", "index.php?func=savesensor&lang=$language");
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Sensor Name"));
		$t->setVariable("VALUE_SENSOR_NAME", "");
		$t->setVariable("TEXT_SENSOR_USER", gettext("Sensor Username"));	
		$t->setVariable("VALUE_SENSOR_USER", "");
		$t->setVariable("TEXT_SENSOR_IP", gettext("Sensor IP"));	
		$t->setVariable("VALUE_SENSOR_IP", "");
		$t->setVariable("TEXT_SENSOR_COMMENT", gettext("Comment"));	
		$t->setVariable("VALUE_SENSOR_COMMENT", "");
		$t->setVariable("TEXT_SENSOR_PASSWORD", gettext("Sensor Password"));	
		$t->setVariable("VALUE_SENSOR_PASSWORD", "");
		$t->setVariable("SENSOR_IDX", "");
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 	 
	 /* Main Function: saveeditedsensor */
	 
	 case "savesensor":

	 	# Cancel was pressed for edit
		if (isset($_POST['cancel_x']) && !empty($_POST['sensor_idx'])) {
			header("Location: $mainServerURL?func=detailssensor&sensor_idx=$_POST[sensor_idx]&lang=$language");	
			exit;
		}
		
		# Cancel was pressed for add
		elseif (isset($_POST['cancel_x']) && empty($_POST['sensor_idx'])) {
			header("Location: $mainServerURL?func=listsensors&lang=$language");	
			exit;
		}
		 
	 	# Sensor was edited with no password change
		if (empty($_POST['sensor_password']) && !empty($_POST['sensor_idx'])) {
			$sensor_idx = $_POST['sensor_idx'];
			
			$query = sprintf($queries['update_sensor_nopw'], 
				$db->escape($_POST['sensor_name']),
				$db->escape($_POST['sensor_user']),
				$db->escape($_POST['sensor_ip']),
				$db->escape($_POST['sensor_comment']),
				$db->escape($sensor_idx)
			);
	 	}
	 	
	 	# Sensor was edited with password change
	 	elseif (!empty($_POST['sensor_password']) && !empty($_POST['sensor_idx'])) {
	 		$sensor_idx = $_POST['sensor_idx'];
	 		
	 		$query = sprintf($queries['update_sensor_pw'], 
				$db->escape($_POST['sensor_name']),
				$db->escape($_POST['sensor_user']),
				$db->escape($_POST['sensor_ip']),
				$db->escape($_POST['sensor_comment']),
				$db->escape($_POST['sensor_password']),
				$db->escape($sensor_idx)	 			
	 		);
	 	}

	 	# Sensor was added
		elseif (empty($_POST['sensor_idx'])) {
			$sensor_idx = $db->nextid("sensor");
			
	 		$query = sprintf($queries['insert_sensor'], 
				$db->escape($_POST['sensor_name']),
				$db->escape($_POST['sensor_user']),
				$db->escape($_POST['sensor_ip']),
				$db->escape($_POST['sensor_comment']),
				$db->escape($_POST['sensor_password']),
				$sensor_idx	 			
	 		);
	 	}
	 	
		$res = $db->query($query);
		
		header("Location: $mainServerURL?func=detailssensor&sensor_idx=$sensor_idx&lang=$language");
		exit;	
		
	 break;

	  /* Main Function: deleteconfirmsensor */
	 
	 case "deleteconfirmsensor":
	 
		printHeader();

		$query = sprintf($queries['select_sensor'], $db->escape($_GET['sensor_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("deleteconfirmsensor.php", true, true); 

		$t->setVariable("TEXT_USER_DELETECONFIRM", gettext("Really delete sensor?"));
		$t->setVariable("URL_POST", "index.php?func=deletesensor&lang=$language");
		$t->setVariable("TEXT_SENSOR_NAME", gettext("Sensor Name"));
		$t->setVariable("SENSOR_NAME", $row['sensor_name']);
		$t->setVariable("TEXT_SENSOR_USER", gettext("Sensor Username"));	
		$t->setVariable("SENSOR_USER", $row['sensor_user']);
		$t->setVariable("TEXT_SENSOR_IP", gettext("Sensor IP"));	
		$t->setVariable("SENSOR_IP", $row['sensor_ip']);
		$t->setVariable("TEXT_SENSOR_COMMENT", gettext("Comment"));	
		$t->setVariable("SENSOR_COMMENT", $row['sensor_comment']);
		$t->setVariable("TEXT_CHOOSE_DELETE_REASSIGN", gettext("Please choose what should be done with recorded attacks related to this sensor"));	
		$t->setVariable("SENSOR_IDX", $row['sensor_idx']);
		$t->setVariable("TEXT_DELETE_INSTANCES", gettext("Delete related attacks"));	
		$t->setVariable("TEXT_REASSIGN_INSTANCES", gettext("Assign to different sensor"));	

		$query = sprintf($queries['select_sensors_with_exception'], $row['sensor_idx']); 
		$res = $db->query($query);
		
		while ($row = $res->fetchRow(MDB2_FETCHMODE_ASSOC)) {
			$t->setCurrentBlock("sensors");
			$t->setVariable("TEXT_NEW_SENSOR_NAME", $row["sensor_name"]);
			$t->setVariable("new_sensor_idx", $row["sensor_idx"]);
			$t->parseCurrentBlock("sensors");	
		}
				
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();

		printFooter();
		
	 break;

	 /* Main Function: deleteuser */
	 
	 case "deletesensor":

	 	# Cancel was pressed for delete
	 	if (isset($_POST['cancel_x']) && !empty($_POST['sensor_idx'])) {
	 		header("Location: $mainServerURL?func=detailssensor&sensor_idx=$_POST[sensor_idx]&lang=$language");
	 		exit;
	 	}

	 	# Deletion was confirmed - reassign all instances
	 	elseif (isset($_POST['confirm_x']) && $_POST['delete_reassign'] == "reassign") {
	 		$query = sprintf($queries['update_instances_sensor_idx'],
	 			$db->escape($_POST['new_sensor']),
	 			$db->escape($_POST['sensor_idx'])
	 		);
	 		
	 		$db->query($query);
	 		
			$query = sprintf($queries['delete_sensor'], 
				$db->escape($_POST['sensor_idx'])
			);
		 	
			$db->query($query);
	 	}
	 	
	 	# Deletion was confirmed - delete all instances and clean up samples
	 	elseif (isset($_POST['confirm_x']) && $_POST['delete_reassign'] == "delete") {
	 		$query = sprintf($queries['select_instances_by_sensor_idx'], 
	 			$db->escape($_POST['sensor_idx']));
	 		
	 		$res = $db->query($query);

	 		while ($row = $res->fetchRow(MDB2_FETCHMODE_ASSOC)) {
	 			$instances[$row['sample_sha512']][] = $row['instance_idx'];
	 		}
	 		
	 		foreach ($instances as $sample_sha512 => $instance_array) {
	 			$query = sprintf($queries["select_sample_details"], $sample_sha512);
	 			
	 			$res = $db->query($query);
	 			$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
	 			
	 			# Calculate new hitcount - it should decrease as instances are deleted
	 			$new_hitcount = $row['sample_hitcount'] - sizeof($instance_array);
	 			
	 			# If all instances for a sample have been deleted, also purge sample and
	 			# sample_info for it
	 			if ($new_hitcount <= 0) {
	 				$query = sprintf($queries['delete_sample'], $sample_sha512);
	 				$db->query($query);

	 				$query = sprintf($queries['delete_sample_info'], $sample_sha512);
	 				$db->query($query);

	 				unlink("$mainSamplePath/$sample_sha512");
	 			}
	 			else {
	 				$query = sprintf($queries['update_sample_hitcount'], $new_hitcount);
	 				$db->query($query);
	 			}
	 			
	 			for ($i=0; $i<count($instance_array); $i++) {
	 				$query = sprintf($queries['delete_instance_by_instance_idx'], $instance_array[$i]);
	 				$db->query($query);
	 				
	 				$query = sprintf($queries['delete_instance_info_by_instance_idx'], $instance_array[$i]);
	 				$db->query($query);	 				
	 			}
	 		}
	 	}
 		else {
 			header("Location: $mainServerURL?func=detailssensor&sensor_idx=$_POST[sensor_idx]&lang=$language");
 			exit;
	 	}
	 	
	 	$query = sprintf($queries['delete_sensor'], $_POST['sensor_idx']);
	 	$db->query($query);
	 	
		header("Location: $mainServerURL?func=listsensors&lang=$language");
		exit;	
		
	 break;
	 	 
	 /* Main Function: listusers */
	 
	 case "listusers":
	 
		printHeader();
	 	
		# Set default values for ordering and sorting of query
		if (!isset($_GET['oby']) || empty($_GET['oby'])) {
			$oby = 'user_idx';
		} 
		else {
			$oby = $db->escape($_GET['oby']);
		}
		
		if (isset($_GET['sort']) && $_GET['sort'] != 'desc') {
			$sort = 'asc';
		}
		else {
			$sort = 'desc';
		}
		
		$query = $queries['select_count_users'];
		$num_db_entries = $db->queryOne($query);
		
		$pagerParameters = array(
			'oby' => $oby,
			'sort' => $sort
		);
	
		$pagerOptions = array(
			'mode'    => 'Sliding',
	        'perPage' => $mainHashesPerPage,
			'delta' => 1,
			'totalItems' => $num_db_entries,
		);
		
		$query = sprintf($queries['select_users'], $oby, $sort);
		$paged_data = Pager_Wrapper_MDB2($db, $query, $pagerOptions);	
	
		$pager1 = & Pager::factory($pagerParameters);
	
		$pager = & Pager::factory($pagerOptions);
		
		$links = $pager->getLinks();
			
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("listusers.php", false, true); 
	
		$t->setVariable("PAGER_LINKS", $pager->links);
		$t->setVariable("URL_LISTUSERS_BY_LAST_LOGIN_UP", "index.php?func=listusers&oby=user_last_login&sort=asc&lang=$language");
		$t->setVariable("URL_LISTUSERS_BY_LAST_LOGIN_DOWN", "index.php?func=listusers&oby=user_last_login&sort=desc&lang=$language");
						
		$t->setVariable("TEXT_USER_NAME", gettext("Username"));
		$t->setVariable("TEXT_USER_LAST_LOGIN", gettext("Last Login"));	
		$t->setVariable("TEXT_USER_COMMENT", gettext("Comment"));
		$t->setVariable("URL_ADD_USER", "index.php?func=adduser&lang=$language");
			
		foreach ($paged_data['data'] as $key => $row) {	
			$t->setCurrentBlock("users");

			$t->setVariable("URL_ADD_USER", "index.php?func=adduser&lang=$language");
			$t->setVariable("URL_DETAIL_USER", "index.php?func=detailsuser&user_idx=$row[user_idx]&lang=$language");
			$t->setVariable("URL_DELETE_USER", "index.php?func=deleteconfirmuser&user_idx=$row[user_idx]&lang=$language");
			$t->setVariable("URL_EDIT_USER", "index.php?func=edituser&user_idx=$row[user_idx]&lang=$language");			
			$t->setVariable("user_name", $row["user_name"]);
			$t->setVariable("user_last_login", date($mainDateformat, $row['user_last_login']));
			$t->setVariable("user_comment", $row["user_comment"]);

			$t->parseCurrentBlock("users");	
		}
			
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;

	 /* Main Function: detailsuser */
	 
	 case "detailsuser":
	 
		printHeader();

		$query = sprintf($queries['select_user'], $db->escape($_GET['user_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("detailsuser.php", true, true); 

		$t->setVariable("URL_DELETE_USER", "index.php?func=deleteconfirmuser&user_idx=$row[user_idx]&lang=$language");
		$t->setVariable("URL_EDIT_USER", "index.php?func=edituser&user_idx=$row[user_idx]&lang=$language");
		$t->setVariable("TEXT_USER_DETAILS", gettext("Details for User"));
		$t->setVariable("TEXT_USER_NAME", gettext("Username"));
		$t->setVariable("USER_NAME", $row['user_name']);
		$t->setVariable("TEXT_USER_COMMENT", gettext("User Comment"));	
		$t->setVariable("USER_COMMENT", $row['user_comment']);
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 	 
	 /* Main Function: edituser */
	 
	 case "edituser":
	 
		printHeader();

		$query = sprintf($queries['select_user'], $db->escape($_GET['user_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("editadduser.php", true, true); 

		$t->setVariable("URL_POST", "index.php?func=saveuser&lang=$language");
		$t->setVariable("TEXT_USER_ADDEDIT", gettext("Edit User"));
		$t->setVariable("TEXT_USER_NAME", gettext("Username"));
		$t->setVariable("VALUE_USER_NAME", $row['user_name']);
		$t->setVariable("TEXT_USER_COMMENT", gettext("User Comment"));	
		$t->setVariable("VALUE_USER_COMMENT", $row['user_comment']);
		$t->setVariable("TEXT_USER_PASSWORD", gettext("User Password"));	
		$t->setVariable("USER_IDX", $row['user_idx']);
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 
	 /* Main Function: adduser */
	 
	 case "adduser":
	 
		printHeader();

		$user_idx = $db->nextid("user");
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("editadduser.php", true, true); 

		$t->setVariable("URL_POST", "index.php?func=saveuser&lang=$language");
		$t->setVariable("TEXT_USER_ADDEDIT", gettext("Add User"));
		$t->setVariable("TEXT_USER_NAME", gettext("Username"));
		$t->setVariable("VALUE_USER_NAME", "");
		$t->setVariable("TEXT_USER_COMMENT", gettext("User Comment"));	
		$t->setVariable("VALUE_USER_COMMENT", "");
		$t->setVariable("TEXT_USER_PASSWORD", gettext("User Password"));	
		$t->setVariable("USER_IDX", "");
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();
		
		printFooter();
		
	 break;
	 	 
	 /* Main Function: saveuser */
	 
	 case "saveuser":
	 	# Cancel was pressed for edit
	 	if (isset($_POST['cancel_x']) && !empty($_POST['user_idx'])) {
	 		header("Location: $mainServerURL?func=detailsuser&user_idx=$_POST[user_idx]&lang=$language");
	 		exit;
	 	}
	 	
		# Cancel was pressed for add
		elseif (isset($_POST['cancel_x']) && empty($_POST['user_idx'])) {
			header("Location: $mainServerURL?func=listusers&lang=$language");
			exit;	
		}
			 	
	 	# User was edited with no password change
	 	if (empty($_POST['user_password']) && !empty($_POST['user_idx'])) {
	 		$user_idx = $_POST['user_idx'];
	 		
			$query = sprintf($queries['update_user_nopw'], 
				$db->escape($_POST['user_name']),
				$db->escape($_POST['user_comment']),
				$db->escape($user_idx)
			);
	 	}
	 	
	 	# User was edited with password change
	 	elseif (!empty($_POST['user_password']) && !empty($_POST['user_idx'])) {
	 		$user_idx = $_POST['user_idx'];
	 		
	 		$query = sprintf($queries['update_user_pw'], 
				$db->escape($_POST['user_name']),
				$db->escape($_POST['user_comment']),
				$db->escape($_POST['user_password']),
				$db->escape($user_idx)	 			
	 		
	 		);	
	 	}

	 	# User was added 
	 	elseif (empty($_POST['user_idx'])) {
	 		$user_idx = $db->nextid("user");
	 		
			$query = sprintf($queries['insert_user'], 
				$db->escape($_POST['user_name']),
				$db->escape($_POST['user_comment']),
				$db->escape($_POST['user_password']),
				$user_idx
			);
	 	}
	 	
		$res = $db->query($query);
		
		header("Location: $mainServerURL?func=detailsuser&user_idx=$user_idx&lang=$language");	
		exit;
		
	 break;

	  /* Main Function: deleteconfirmuser */
	 
	 case "deleteconfirmuser":
	 
		printHeader();

		$query = sprintf($queries['select_user'], $db->escape($_GET['user_idx']));
		
		$res = $db->query($query);
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("deleteconfirmuser.php", true, true); 

		$t->setVariable("URL_POST", "index.php?func=deleteuser&lang=$language");
		$t->setVariable("TEXT_USER_DELETECONFIRM", gettext("Really delete User?"));
		$t->setVariable("TEXT_USER_NAME", gettext("Username"));
		$t->setVariable("USER_NAME", $row['user_name']);
		$t->setVariable("TEXT_USER_COMMENT", gettext("User Comment"));	
		$t->setVariable("USER_COMMENT", $row['user_comment']);
		$t->setVariable("USER_IDX", $row['user_idx']);
		
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();

		printFooter();
		
	 break;

	 /* Main Function: deleteuser */
	 
	 case "deleteuser":

	 	# Cancel was pressed for delete
	 	if (isset($_POST['cancel_x']) && !empty($_POST['user_idx'])) {
	 		header("Location: $mainServerURL?func=detailsuser&user_idx=$_POST[user_idx]&lang=$language");
	 		exit;
	 	}

	 	else {
			$query = sprintf($queries['delete_user'], 
				$db->escape($_POST['user_idx'])
			);
		 	
			$db->query($query);
			
			header("Location: $mainServerURL?func=listusers&lang=$language");	
			exit;
	 	}
	 break;
	 
	 /* Main Function: stats */
	
	 case "stats":
		printHeader();
					
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("stats.php", true, true); 
		
		$t->setVariable("TEXT_STATS_INSTANCES", gettext("Statistics for Attacks"));
		$t->setVariable("TEXT_STATS_SAMPLES", gettext("Statistics for Unique Samples"));
		
	 	$t->setCurrentBlock();
		$t->parseCurrentBlock();
		
		$t->show();

		printFooter();		
	 break;
	/*  Main Function to Make Maps Using Afterglow */	
	 case "visual":
		printHeader();
		require_once 'gmap.php';
		printFooter();
	 break;

	/*  Main Function to Make Maps Using Afterglow */
	 case "mkv":
		printheader();
		$t = new HTML_Template_IT("./templates"); 
		$t->loadTemplatefile("mkv.php", false, true); 

		if($_GET['tag'] == 1)		
		{
			
		}
		else if ($_GET['tag'] == 2)
		{	
			$csvpath = $tmplocation.md5($auth->session['username']).'.csv';		
			$query = sprintf($queries['csv_make2'],$db->escape($_GET['base']),$csvpath);
			$res = $db->query($query);
			exec ('cat '.$csvpath.' | perl '.$mainafterglowpath.' -c '.$mainvisualpath.'color.properties | '.$gviz.' -Tjpg -o '.$mainvisualpath.md5($auth->session['username']).'2.jpg');
			echo exec('rm -rf '.$csvpath);			
			$t->setVariable("TEXT_IMAGE", gettext("Afterglow Mapping for ASN IP and Hash"));
			$t->setVariable("TEXT_IMAGE_PATH", $mainvisualpathsrv.md5($auth->session['username']).'2.jpg');
			
		}
		else if ($_GET['tag'] == 3)
		{
			$csvpath = $tmplocation.md5($auth->session['username']).'.csv';		
			$query = sprintf($queries['csv_make1'],$db->escape($_GET['base']),$csvpath);
			$res = $db->query($query);
			exec ('cat '.$csvpath.' | perl '.$mainafterglowpath.' -c '.$mainvisualpath.'color.properties | '.$gviz.' -Tjpg -o '.$mainvisualpath.md5($auth->session['username']).'3.jpg');
			echo exec('rm -rf '.$csvpath);			
			$t->setVariable("TEXT_IMAGE", gettext("Afterglow Mapping for Country IP and Hash"));
			$t->setVariable("TEXT_IMAGE_PATH", $mainvisualpathsrv.md5($auth->session['username']).'3.jpg');
		}
		else
		{
			echo 'Dont mess with tag field';
		}
		$t->setCurrentBlock();
		$t->parseCurrentBlock();
		$t->show();
		printFooter();

	 break;

	
	 /* Main Function: logout */
	
	 case "logout":
	 	$auth->logout();
	 	header("Location: $mainServerURL");
	 break;
	 
	}

	$db->disconnect();
}
	
ob_end_flush();
?>
