<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

function exists_sha512($params) {
	global $logger;
	global $db;
	global $queries;
	global $mainSamplePath;
	
	$logger->log("Initiating known binary check", PEAR_LOG_NOTICE);

	$logger->log("Initiating known binary check for sha512 hash $params[sha512]", PEAR_LOG_DEBUG);
	
	$query = sprintf($queries['select_sample_details'], $db->escape($params['sha512']));
	
	$res = $db->query($query);
	
	if (PEAR::isError($res)) {
	        $logger->log("DB error during known binary check:", PEAR_LOG_EMERG); 
	        $logger->log($res->getMessage(), PEAR_LOG_EMERG);
	        echo "S_ERROR";
	        exit;
	}

	if ($res->numRows() == 1 || file_exists("$mainSamplePath/$params[sha512].lck")) {

		# Determine if binary is in DB or a lockfile is present/binary is bein processed)
	    if ($res->numRows() == 1) {
        	$logger->log("Known binary found in DB with SHA512 hash $_POST[sha512]", PEAR_LOG_DEBUG);

        }
        elseif (file_exists("$mainSamplePath/$params[sha512].lck")) {	
        	$logger->log("Lockfile for binary found with SHA512 hash $_POST[sha512]", PEAR_LOG_DEBUG);
        }

        $row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
        
		$query = sprintf($queries['sample_hitcount_increment'], time(), $row['sample_hitcount']+1, $params['sha512']);

		$db->query($query);
		
		return true;
	}
	else {
		return false;
	}	
}

function loginFunction ($username = null, $status = null, $auth) {	
	global $mainVersion;
	$t = new HTML_Template_IT("./templates"); 
	$t->loadTemplatefile("login.php", true, false); 	
	$t->setVariable("VERSION", $mainVersion);
	$t->setVariable("PLEASE_LOGIN", gettext("Please Login"));
	$t->setVariable("USERNAME", gettext("Username"));
	$t->setVariable("PASSWORD", gettext("Password"));
	$t->setVariable("SUBMIT", gettext("Login"));
	$t->parse();
	$t->show();
}

function processIP($ip_string) {
	global $mainIpObfuscationLevel;

	$i = $mainIpObfuscationLevel;

	#$ip_string = long2ip($ip_decimal);

	$ip_bytes = explode(".", $ip_string);

	if ($i == '1') { $ip_bytes[3] = "X"; }
	if ($i == '2') { $ip_bytes[3] = "X"; $ip_bytes[2] = "X"; }
	if ($i == '3') { $ip_bytes[3] = "X"; $ip_bytes[2] = "X"; $ip_bytes[1] = "X"; }
	if ($i == '4') { $ip_bytes[3] = "X"; $ip_bytes[2] = "X"; $ip_bytes[1] = "X"; $ip_bytes[0] = "X"; }

	return implode(".", $ip_bytes);
}

function processUrl($url) {
	preg_match("/([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})/", $url, $matches);
	// Do we have something in the URL that looks like an IP address?
	if(isset($matches[0])) {
		$ip_bytes = explode(".", $matches[0]);

		// Is it _really_ an IP address?
		for ($i=0 ; $i < 4 ; $i++) {
			if ($ip_bytes[$i] > 255) {
		              return $url;
			}
		}

		// Its an IP, all right
		#$ip_string = processIp(ip2long($matches[0]));
		$ip_string = processIp($matches[0]);
		
		$new_url = preg_replace("/$matches[0]/", $ip_string, $url);
	}
	else {
		$new_url = $url;
	}

	return $new_url;
}

function printHeader() {
	global $mainVersion;
	global $language;
	
	$t = new HTML_Template_IT("./templates"); 
	$t->loadTemplatefile("header.php", true, true); 
	$t->setVariable("VERSION", $mainVersion);
	$t->setVariable("URL_LISTHITS", "index.php?func=listinstances&lang=$language");
	$t->setVariable("TEXT_HITS", gettext("Hits"));
	$t->setVariable("URL_LISTSAMPLES", "index.php?func=listsamples&lang=$language");
	$t->setVariable("TEXT_MALWARE", gettext("Malware"));
	$t->setVariable("URL_STATS", "index.php?func=stats&lang=$language");
	$t->setVariable("TEXT_STATS", gettext("Stats"));
	$t->setVariable("URL_USERS", "index.php?func=listusers&lang=$language");
	$t->setVariable("TEXT_USERS", gettext("Users"));	
	$t->setVariable("URL_SENSORS", "index.php?func=listsensors&lang=$language");
	$t->setVariable("TEXT_SENSORS", gettext("Sensors"));		
	$t->setVariable("URL_VISUAL", "index.php?func=visual&lang=$language");
	$t->setVariable("TEXT_VISUAL", gettext("GoogleMaps"));
	$t->setVariable("URL_LOGOUT", "index.php?func=logout&lang=$language");
	$t->setVariable("TEXT_LOGOUT", gettext("Logout"));	
	$t->parse();
	$t->show();
}

function printFooter() {
	$t = new HTML_Template_IT("./templates"); 
	$t->loadTemplatefile("footer.php", false, false); 
	$t->parse();
	$t->show();
}

function xml2array($contents, $get_attributes=1) {
	/**
	 * xml2array() will convert the given XML text to an array in the XML structure.
	 * Link: http://www.bin-co.com/php/scripts/xml2array/
	 * Arguments : 
	 * $contents - The XML text
	 * $get_attributes - 1 or 0. If this is 1 the function will get the attributes as well as the tag values 
	 * - this results in a different array structure in the return value.
	 * Return: The parsed XML in an array form.
	 */
	
    if(!$contents) return array();

    if(!function_exists('xml_parser_create')) {
        //print "'xml_parser_create()' function not found!";
        return array();
    }
    //Get the XML parser of PHP - PHP must have this module for the parser to work
    $parser = xml_parser_create();
    xml_parser_set_option( $parser, XML_OPTION_CASE_FOLDING, 0 );
    xml_parser_set_option( $parser, XML_OPTION_SKIP_WHITE, 1 );
    xml_parse_into_struct( $parser, $contents, $xml_values );
    xml_parser_free( $parser );

    if(!$xml_values) return;//Hmm...

    //Initializations
    $xml_array = array();
    $parents = array();
    $opened_tags = array();
    $arr = array();

    $current = &$xml_array;

    //Go through the tags.
    foreach($xml_values as $data) {
        unset($attributes,$value);//Remove existing values, or there will be trouble

        //This command will extract these variables into the foreach scope
        // tag(string), type(string), level(int), attributes(array).
        extract($data);//We could use the array by itself, but this cooler.

        $result = '';
        if($get_attributes) {//The second argument of the function decides this.
            $result = array();
            if(isset($value)) $result['value'] = $value;

            //Set the attributes too.
            if(isset($attributes)) {
                foreach($attributes as $attr => $val) {
                    if($get_attributes == 1) $result['attr'][$attr] = $val; //Set all the attributes in a array called 'attr'
                    /**  :TODO: should we change the key name to '_attr'? Someone may use the tagname 'attr'. Same goes for 'value' too */
                }
            }
        } elseif(isset($value)) {
            $result = $value;
        }

        //See tag status and do the needed.
        if($type == "open") {//The starting of the tag '<tag>'
            $parent[$level-1] = &$current;

            if(!is_array($current) or (!in_array($tag, array_keys($current)))) { //Insert New tag
                $current[$tag] = $result;
                $current = &$current[$tag];

            } else { //There was another element with the same tag name
                if(isset($current[$tag][0])) {
                    array_push($current[$tag], $result);
                } else {
                    $current[$tag] = array($current[$tag],$result);
                }
                $last = count($current[$tag]) - 1;
                $current = &$current[$tag][$last];
            }

        } elseif($type == "complete") { //Tags that ends in 1 line '<tag />'
            //See if the key is already taken.
            if(!isset($current[$tag])) { //New Key
                $current[$tag] = $result;

            } else { //If taken, put all things inside a list(array)
                if((is_array($current[$tag]) and $get_attributes == 0)//If it is already an array...
                        or (isset($current[$tag][0]) and is_array($current[$tag][0]) and $get_attributes == 1)) {
                    array_push($current[$tag],$result); // ...push the new element into that array.
                } else { //If it is not an array...
                    $current[$tag] = array($current[$tag],$result); //...Make it an array using using the existing value and the new value
                }
            }

        } elseif($type == 'close') { //End of tag '</tag>'
            $current = &$parent[$level-1];
        }
    }

    return($xml_array);
} 

function incrementStats($stat_key, $stat_value) {
	global $logger;
	global $db;
	global $queries;

	# Check if stat entry exists
	$query = sprintf($queries['select_stat_by_key_value'], $stat_key, $stat_value);
	$res = $db->query($query);
	
	# Entry does not exist - use insert
	if ($res->numRows() == 0) {
		$query = sprintf($queries['insert_stat'], 
			$db->nextID('stat'), 
			$db->escape($stat_key), 
			$db->escape($stat_value), 
			1,
			time());
			
		$db->query($query);
	}
	else {
		$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
		$counter = $row['stat_counter']+1;
		
		$query = sprintf($queries['increment_stat'], 
			$db->escape($counter),
			time(), 
			$db->escape($stat_key), 
			$db->escape($stat_value));
			
		$db->query($query);
	}
	
	return;
}

function updateRRD ($stat_key, $stat_value, $stat_counter) {
	global $mainStats;
	global $logger;
	
	$logger->log("updateRRD received parameters: $stat_key, $stat_value, $stat_counter", PEAR_LOG_DEBUG);
	
	$rrd_file = sprintf("rrd/%s_%s.rrd", $stat_key, $stat_value);
	$rrd_image = sprintf("images/%s_%s.png", $stat_key, $stat_value);
	
	# Check if RRD file exists and is enabled, if not create the RRD
	if (!file_exists($rrd_file)) {
		$logger->log("Creating RRD file: $rrd_file", PEAR_LOG_DEBUG);
		
		$rrd_options = $mainStats[$stat_key]['options'];
		
		$logger->log("RRD options:", PEAR_LOG_DEBUG);
		$logger->log(print_r($rrd_options, TRUE), PEAR_LOG_DEBUG);
		
		$ret = rrd_create($rrd_file, $rrd_options, count($rrd_options));
		
		if ($ret == 0) {
			$error = rrd_error();
			$logger->log("RRD error: $error", PEAR_LOG_ERROR);
		}
	}
	
	# Update RRD and graph
	$ret = rrd_update($rrd_file, "N:$stat_counter");

	if ($ret == 0) {
		$error = rrd_error();
		$logger->log("RRD update error: $error", PEAR_LOG_ERROR);
	}	
	
	$graph_options = array( "--start", "-1d",
                "DEF:incoming=$rrd_file:input:AVERAGE",
                "AREA:incoming#00FF00:Incoming\\n",
                "GPRINT:incoming:LAST:Last\: %6.0lf %S",
                "GPRINT:incoming:AVERAGE:Average\: %6.0lf %s",
                "GPRINT:incoming:MAX:Max\: %6.0lf %s",
                "GPRINT:incoming:MIN:Min\: %6.0lf %s"
	);
	
//	$logger->log("Creating graph for $stat_key", PEAR_LOG_ERROR);
	
	$ret = rrd_graph("$rrd_image", $graph_options, count($graph_options));
	
	if ($ret == 0) {
		$error = rrd_error();
		$logger->log("RRD graph error: $error", PEAR_LOG_ERROR);
	}	
}
?>
