<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

/*
 * Module "asn":
 * this module takes a decimal ip address as parameter and
 * retrieves the autonomous system number the ip "belongs" to.
 * 
 * Retrieval is done by querying the nameserver 
 * origin.asn.cymru.com for a TXT record
 * 
 * The lookup service is provided by the Team Cymru project:
 * http://www.cymru.com/ 
 * 
 */

require_once 'Net/DNS.php';
require_once 'Log.php';
require_once 'HTML/Template/IT.php';

class ASN {
	
	var $classname = 'ASN';
	
	var $info_type = 'other';
	
	var $info_name = 'asn';
		
	var $pattern_result = '^[^0-9]*([0-9]+) \| (.*) \| .* \| (.*) \| .*$';
	
	var $result_xml = '<?xml version="1.0" encoding="UTF-8"?><asn><as_number>\\1</as_number><network_cidr>\\2</network_cidr><registry>\\3</registry></asn>';
	
	function process($params) {
		global $logger;

		$ip_decimal = $params['source_host'];
		
		$ip_string = long2ip($ip_decimal);
		
		$bytes = explode('.', $ip_string);
		
		$ip_reverse = sprintf('%s.%s.%s.%s',
			$bytes[3], $bytes[2], $bytes[1], $bytes[0]);

		$resolver = new Net_DNS_Resolver();
		
		$response = $resolver->query("$ip_reverse.origin.asn.cymru.com", 'TXT');
		
		$rr = $response->answer;
		
		$result_string = $rr[0]->rdata;
		
		$logger->log("Received result from $this->classname", PEAR_LOG_INFO);
		
		$logger->log("Received result from $this->classname: $result_string", PEAR_LOG_DEBUG);
		
		$instance_info_details = ereg_replace($this->pattern_result, $this->result_xml, $result_string);
		
		$logger->log("Returning instance_info_details in $this->classname: $instance_info_details", PEAR_LOG_DEBUG);
		
		# Return empty string if regular expression matching fails
		if ($instance_info_details == $result_string) {
			$instance_info_details = "";
		}
		
		return $instance_info_details;
	}
	
	function render($instance_info_details) {
		global $mainGettextdomain;
		bindtextdomain($mainGettextDomain, "modules/$this->info_name/locale");
		textdomain($mainGettextDomain);
		
		$t = new HTML_Template_IT("modules/$this->info_name"); 
		$t->loadTemplatefile("template.php", false, true); 
		
		$t->setVariable("TEXT_ASN_DETAILS",gettext("Autonomous System Details"));
		
		#$xml_data = xmlStructToArray($instance_info_details);
		$xml_data = xml2array($instance_info_details);
		
		$t->setVariable("TEXT_ASN",gettext("Autonomous System Number"));
		$t->setVariable("ASN", $xml_data[$this->info_name]['as_number']['value']);

		$t->setVariable("TEXT_NETWORK",gettext("Network"));
		$t->setVariable("NETWORK", $xml_data[$this->info_name]['network_cidr']['value']);

		$t->setVariable("TEXT_REGISTRY",gettext("IP Registry"));
		$t->setVariable("REGISTRY", $xml_data[$this->info_name]['registry']['value']);
		
		$t->setVariable("TEXT_VISUAL_ASN",gettext("Map of ASN"));
		$t->setVariable("URL_VISUAL_ASN", "index.php?func=mkv&base=" . $xml_data[$this->info_name]['as_number']['value'] . "&tag=2&lang=$language");
	
		$t->setVariable("TEXT_ROBTEX_ASN",gettext("Links to Robtex"));
		$t->setVariable("URL_ROBTEX_ASN", "http://www.robtex.com/as/as" . $xml_data[$this->info_name]['as_number']['value'] . ".html");

		$t->setVariable("TEXT_PHISH_ASN",gettext("Links to Phish Tank"));
		$t->setVariable("URL_PHISH_ASN", "http://www.phishtank.com/asn_search.php?asn=" . $xml_data[$this->info_name]['as_number']['value'] . "&valid=y&active=All&Search=Search");

		return $t->get();
	}
}

?>
