<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

/*
 * Module "geoip":
 * this module takes a decimal ip address as parameter and
 * retrieves the geographical location data for this ip.
 * 
 * Retrieval is done by lookup in the Maxmind GeoIP database 
 * 
 * For details on GeoIP see http://www.maxmind.com/
 * 
 */

require_once 'Net/GeoIP.php';
require_once 'Log.php';
require_once 'HTML/Template/IT.php';

class GEOIP {
	
	/* Module configuration starts here */
	# Set path to GeoIP database file
	var $geoip_location = '/usr/share/GeoIP/GeoLiteCity.dat';
	/* Module configuration ends here */
	
	var $classname = 'GEOIP';
	
	var $info_type = 'other';
	
	var $info_name = 'geoip';
	
	function process($params) {
		global $logger;
		
		$ip_decimal = $params['source_host'];
		
		$ip_string = long2ip($ip_decimal);
		
		$geoip = Net_GeoIP::getInstance($this->geoip_location);
		
		$location = $geoip->lookupLocation($ip_string);

		$logger->log("Received result from $this->classname", PEAR_LOG_INFO);
		
		$logger->log("Received result from $this->classname:", PEAR_LOG_DEBUG);
		$logger->log(print_r($location, TRUE), PEAR_LOG_DEBUG);
		
		$result_pattern = '<?xml version="1.0" encoding="UTF-8"?><geoip>';
		$result_pattern .= '<country_code>%s</country_code><country_code3>%s</country_code3>';
		$result_pattern .= '<country_name>%s</country_name><region>%s</region>';
		$result_pattern .= '<city><![CDATA[%s]]></city><postal_code>%s</postal_code>';
		$result_pattern .= '<latitude>%s</latitude><longitude>%s</longitude>';
		$result_pattern .= '<dma_code>%s</dma_code><area_code>%s</area_code>';
		$result_pattern .= '</geoip>';
		
		$instance_info_details = sprintf($result_pattern,
			$location->countryCode,
			$location->countryCode3,
			$location->countryName,
			$location->region,
			preg_replace('/[^!-%\x27-;=?-~ ]/e', '"&#".ord("$0").chr(59)', $location->city),
			$location->postalCode,
			$location->latitude,
			$location->longitude,
			$location->dmaCode,
			$location->areaCode
		);
	
		return $instance_info_details;
	}
	
	function render($instance_info_details) {
		global $mainGettextdomain;
		bindtextdomain($mainGettextDomain, "modules/$this->info_name/locale");
		textdomain($mainGettextDomain);
				
		$t = new HTML_Template_IT("modules/$this->info_name"); 
		$t->loadTemplatefile("template.php", false, true); 
		
		$t->setVariable("TEXT_GEOIP_DETAILS",gettext("GeoIP Details"));

		$xml_data = xml2array($instance_info_details);
				
		# Check if a PNG image exists for this country - if not use empty image
		$server_path = preg_replace('/^(.*)\/[^\/]*/', '$1', $_SERVER['SCRIPT_FILENAME']);
		$flag_image = sprintf("%s.png", strtolower($xml_data[$this->info_name]['country_code']['value']));
		
		if (!file_exists("$server_path/images-flags/$flag_image")) {
			$xml_data[$this->info_name]['country_code']['value'] = "unknown";
		}

		# Check if city is known
		if (isset($xml_data[$this->info_name]['city']['value']) && !empty($xml_data[$this->info_name]['city']['value'])) {
			$t->setVariable("CITY", $xml_data[$this->info_name]['city']['value']);
		}
		else {
			$t->setVariable("CITY", gettext("Unknown"));
		}
			
		$t->setVariable("TEXT_COUNTRY_NAME",gettext("Country"));
		$t->setVariable("COUNTRY_NAME", $xml_data[$this->info_name]['country_name']['value']);
		$t->setVariable("COUNTRY_ISO", strtolower($xml_data[$this->info_name]['country_code']['value']));
		
		$t->setVariable("TEXT_CITY",gettext("City"));
		
		return $t->get();
	}
}

?>
