<?

/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

/*
 * Module "virustotal":
 * 
 * This module sends an e-mail to scan@virustotal.com with a binary
 * attached. VirusTotal will perform a scan of the binary with multiple
 * AV scanners and return an analysis by email.
 * 
 * The result email will later be read by cron.php, parsed and fed into
 * the database.
 * 
 * For details on VirusTotal see http://www.virustotal.com/
 */

require_once 'Mail.php';
require_once 'Mail/mime.php';
require_once 'Net/POP3.php';
require_once 'Net/SMTP.php';
require_once 'Log.php';
require_once 'HTML/Template/IT.php';

class VIRUSTOTAL {
	
	/* Module configuration starts here */
	var $virustotal_recipient = "";
	var $virustotal_sender = "";	
	var $pop3_server = "";
	var $pop3_port = "";
	var $pop3_username = "";
	var $pop3_password = "";
	var $smtp_server = "";
	var $smtp_auth = "";
	var $smtp_username = "";
	var $smtp_password = "";
	/* Module configuration ends here */
	
	var $classname = 'VIRUSTOTAL';
	
	var $param = 'sha512';
	
	var $info_type = 'mail';
	
	var $info_name = 'virustotal';

	var $subject_pattern = ".VirusTotal. Server notification";
	var $mail_replace_pattern = "";
	
	var $flush_mail = FALSE;
	
	function process($params) {

		global $mainSamplePath;
		global $logger;
		
		$sample_sha512 = $params['sha512'];
		$sample_md5 = $params['md5'];
		
		$headers = array(
			'Return-Path' => $this->virustotal_sender,
			'Date' => date("r"),
			'From'    => "nepenthefe <$this->virustotal_sender>",
			'To'    => "Virustotal <$this->virustotal_recipient>",
			'Subject' => 'SCAN'
			);	
		
		$mime = new Mail_mime("\n");
		
		$mime->addAttachment("$mainSamplePath/$sample_sha512", 'application/octet-stream', $sample_md5, TRUE);
		
		$body = $mime->get();
		$headers = $mime->headers($headers);
		
		$logger->log("Created mail-body for $this->classname", PEAR_LOG_INFO);
		
		$logger->log("Created mail-body for $this->classname:", PEAR_LOG_DEBUG);
		$logger->log(print_r($body, TRUE), PEAR_LOG_DEBUG);		
		
		$logger->log("Created mail-headers for $this->classname", PEAR_LOG_INFO);
		
		$logger->log("Created mail-headers for $this->classname:", PEAR_LOG_DEBUG);
		$logger->log(print_r($headers, TRUE), PEAR_LOG_DEBUG);

		$mail =& Mail::factory('smtp', array("host" => $this->smtp_server, "auth" => $this->smtp_auth, "username" => $this->smtp_username,"password" => $this->smtp_password, "debug" => FALSE));

		$res = $mail->send("$this->virustotal_recipient", $headers, $body);

		if (PEAR::isError($res)) {
			$logger->log("Failed to send mail to $this->virustotal_recipient :", PEAR_LOG_EMERG);
			$logger->log($res->getMessage, PEAR_LOG_EMERG);
	    	return;
		}
	
		else {
			$logger->log("Mail sent to $this->virustotal_recipient", PEAR_LOG_INFO);
		}
		
		return;
	}
	
	function cron() {
		
		global $queries;
		global $logger;
		
		$pop3 =& new Net_POP3();
		
		if(!$pop3->connect($this->pop3_server, $this->pop3_port)) {
			$logger->log("Failed to connect to pop3 server $this->pop3_server", PEAR_LOG_EMERG);
			return false;
		}
		
		$pop3res = $pop3->login($this->pop3_username, $this->pop3_password);
	
		if (PEAR::isError($pop3res)) {
			$logger->log("Failed login to pop3 server $this->pop3_server with username $this->pop3_username", PEAR_LOG_EMERG);
	    	return false;
		}
		
		$id2subject = array();
		$listing = $pop3->getListing();
		
		# Retrieve listing of messages and headers and look for 'Subject' matching the Virustotal pattern
		foreach ($listing as $message) {
			$id = $message['msg_id'];
			$headers = $pop3->getParsedHeaders($id);
			
			$subject_pattern = $this->subject_pattern;
			if (preg_match("/$subject_pattern/", $headers['Subject'])) {
				$matched_ids[] = $id;
			}
		}
		
		# Read mails
		reset($matched_ids);
		
		# Read all mails with matching subject
		for ($i=0; $i<count($matched_ids); $i++) {
			$msg_id = $matched_ids[$i];
			$raw_mail = $pop3->getBody($msg_id);
			$res_array_av = array();
			$res_array_misc = array();
			
			# Split mail content by newline character and store in array
			$results_only = preg_replace("/(\015\012)|(\015)|(\012)/","\n",$raw_mail);
			$results_only = explode("\n", $results_only);

			# Parse lines of matched mails
			for ($j=0; $j<count($results_only); $j++) {
				$res_tmp = "";
				
				# Parse misc infos "[ file data ]" and "[ notes ]"
				# * sha1: 5b3b07f056bb3369241bc91fd159675c486e8c0f
				# packers: Themida
				if (preg_match('/^[\* ]*([\S]+): (.*)$/', $results_only[$j], $res_tmp) == 1 &&
					!empty($res_tmp[1]) && !empty($res_tmp[2])) {
						
					$logger->log("Match for misc: $results_only[$j]", PEAR_LOG_DEBUG);
					$res_array_misc[] = array(
						'type' => trim($res_tmp[1]),
						'value' => trim($res_tmp[2])
					);
				}
				
				# Parse AV results "[ scan results ]
				# AntiVir	7.6.0.75/20080326	found [W32/Virut.AU]
				if (preg_match('/^([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+(\[.*\]|nothing)$/', $results_only[$j], $res_tmp) == 1 &&
					!empty($res_tmp[1]) && !empty($res_tmp[2]) && !empty($res_tmp[4])) {
					$logger->log("Match for av: $results_only[$j]", PEAR_LOG_DEBUG);
					$res_array_av[] = array(
						'vendor' => trim($res_tmp[1]),
						'version' => trim($res_tmp[2]),
						'result' => preg_replace("/[\[\]]*/", "", $res_tmp[4])
					);	
				}
				
				# Parse for MD5 hash (filename)
				if (preg_match('/"(.*)"/', $results_only[$j], $res_tmp) == 1 &&
					!empty($res_tmp[1])) {
					$logger->log("Match for md5: $res_tmp[1]", PEAR_LOG_DEBUG);
					$md5 = $res_tmp[1];	
				}
			}

			# AV info an misc info are extracted, lets create the result array
			$result_xml = '<?xml version="1.0" encoding="UTF-8"?><virustotal>';
			
			for ($k=0; $k<count($res_array_av); $k++) {
				$result_xml .= sprintf("<vendor>%s</vendor><version>%s</version><result>%s</result>",
					$res_array_av[$k]['vendor'],
					$res_array_av[$k]['version'],
					$res_array_av[$k]['result']);
			}
	
			for ($k=0; $k<count($res_array_misc); $k++) {
				$result_xml .= sprintf("<type>%s</type><value>%s</value>",
					$res_array_misc[$k]['type'],
					$res_array_misc[$k]['value']);
			}		
			
			$result_xml .= '</virustotal>';
			
			$res_array[] = array(
				"md5" => $md5,
				"sha512" => '',
				"result_xml" => $result_xml);
		}
		
		return $res_array;
	}
	
	function render($sample_info_details) {
		global $mainGettextdomain;
		bindtextdomain($mainGettextDomain, "modules/$this->info_name/locale");
		textdomain($mainGettextDomain);
				
		if (empty($sample_info_details)) {
			return;
		}
		
		$t = new HTML_Template_IT("modules/$this->info_name"); 
		$t->loadTemplatefile("template.php", false, true); 
		
		$t->setVariable("TEXT_VIRUSTOTAL_DETAILS",gettext("Details from VirusTotal"));
		
		$t->setVariable("TEXT_VIRUSTOTAL_VENDOR",gettext("Vendor"));
		$t->setVariable("TEXT_VIRUSTOTAL_VERSION",gettext("Version"));
		$t->setVariable("TEXT_VIRUSTOTAL_RESULT",gettext("Result"));

		$t->setVariable("TEXT_VIRUSTOTAL_MISC_TYPE",gettext("Info Type"));
		$t->setVariable("TEXT_VIRUSTOTAL_MISC_VALUE",gettext("Value"));
		
		$t->setVariable("DIV_ID", $this->info_name);
		
		$xml_data = xml2array($sample_info_details);
		
		$av_vendors = $xml_data[$this->info_name]['vendor'];
		$av_versions = $xml_data[$this->info_name]['version'];
		$av_results = $xml_data[$this->info_name]['result'];
		
		$misc_types = $xml_data[$this->info_name]['type'];
		$misc_values = $xml_data[$this->info_name]['value'];
		
		# Parse AV results
		for ($i=0; $i<count($av_vendors); $i++) {
			$t->setCurrentBlock("av_results");
			$t->setVariable("AV_VENDOR", $av_vendors[$i]['value']);
			$t->setVariable("AV_VERSION", $av_versions[$i]['value']);
			$t->setVariable("AV_RESULT", $av_results[$i]['value']);	
			
			$t->parseCurrentBlock("av_results");	
		}
		
		# Parse misc results
		for ($i=0; $i<count($misc_types); $i++) {
			$t->setCurrentBlock("misc_results");
			$t->setVariable("MISC_TYPE", $misc_types[$i]['value']);
			$t->setVariable("MISC_VALUE", $misc_values[$i]['value']);
			
			$t->parseCurrentBlock("misc_results");	
		}		
		
		return $t->get();
	}
}

?>
