<?
/*

This file is part of NepenthesFE,
Copyright (C) 2008  Emre Bastuz <info@emre.de> and Harsh R Patel <har.duro@gmail.com>

NepenthesFE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

NepenthesFE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NepenthesFE.  If not, see <http://www.gnu.org/licenses/>.

*/

require_once 'config.php';
require_once 'queries.php';
require_once 'lib/helper_functions.php';
require_once 'MDB2.php';
require_once 'Log.php';

# Initialize logging
$logger = &Log::singleton('file', "$mainLogFile", 'NEPENTHESFE_SUBMIT', array(), constant($mainLogLevel));

if (PEAR::isError($logger)) {
	die("Can not create logfile");
}

# Initialize database
$db =& MDB2::connect($mainDbdsn, $mainDatabaseOptions);

if (PEAR::isError($db)) {
	$logger->log("Failed to connect to database:", PEAR_LOG_PEAR_LOG_EMERG);
	$logger->log($db->getMessage, PEAR_LOG_EMERG);
    exit;
}

# Authenticate sensor with HTTP Basic Auth via DB
$query = sprintf($queries['auth_sensor'], $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], $_SERVER['REMOTE_ADDR']);

$res = $db->query($query);

# A serious DB error occured - log and bail out
if (PEAR::isError($res)) {
        $logger->log("DB error during sensor authentication:", PEAR_LOG_EMERG);
        $logger->log($res->getMessage(), PEAR_LOG_EMERG);
        echo "S_ERROR";
        exit;
}

$row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

# Authentication failed for a sensor
if ($res->numRows() == 0) {
        $logger->log("Authentication failed for sensor $_SERVER[REMOTE_ADDR] with username $_SERVER[PHP_AUTH_USER]", 
                PEAR_LOG_WARNING);
        echo "S_ERROR";
        exit;
}

# Remember sensor details
$sensor['sensor_idx'] = $row['sensor_idx'];
$sensor['sensor_name'] = $row['sensor_name'];
$sensor['sensor_user'] = $row['sensor_user'];
$sensor['sensor_ip'] = $row['sensor_ip'];

# Log received POST parameters for debugging purposes
$logger->log("Received POST parameters after successful sensor authentication:", PEAR_LOG_DEBUG);
$logger->log(var_export($_POST, TRUE), PEAR_LOG_DEBUG);
$logger->log(var_export($_FILES, TRUE), PEAR_LOG_DEBUG);

# Determine if instance or sample have been transmitted
if (isset($_FILES['file'])) {
	$logger->log('File detected - setting function "sample"', PEAR_LOG_DEBUG);
	$funct = "sample";
}
elseif (isset($_POST['source_host'])) {
	$logger->log('Post parameter source_host detected - assuming func "instance"', PEAR_LOG_DEBUG);
	$funct = "instance";
}
else {
	$logger->log('Unable to detect func - exiting', PEAR_LOG_DEBUG);
	exit;
}

# Check if all required POST parameters have been transmitted
if ($funct == "instance") {
	if (!isset($_POST['email'])) { $missing_param[] = "email"; }	
	if (!isset($_POST['url'])) { $missing_param[] = "url"; }
	if (!isset($_POST['trigger'])) { $missing_param[] = "trigger"; }
	if (!isset($_POST['filetype'])) { $missing_param[] = "filetype"; }
	if (!isset($_POST['source_host'])) { $missing_param[] = "source_host"; }
	if (!isset($_POST['target_host'])) { $missing_param[] = "target_host"; }
	if (!isset($_POST['filename'])) { $missing_param[] = "filename"; }
	if (!isset($_POST['md5'])) { $missing_param[] = "md5"; }
	if (!isset($_POST['sha512'])) { $missing_param[] = "sha512"; }
	
	# Parameters missing - log and exit
	if (isset($missing_param) && is_array($missing_param)) {
		$log_tmp = sprintf('Submission parameters missing for "instance": %s',
			implode(" ", $missing_param)
		);
			
		$logger->log($log_tmp, PEAR_LOG_WARNING);
		echo "S_ERROR";
		exit;
	}
	
	# Parameters are fine - save in vars
	else {
        $params['email'] = $_POST['email'];
        $params['url'] = $_POST['url'];
        $params['trigger'] = $_POST['trigger'];
        $params['filetype'] = $_POST['filetype'];
        $params['source_host'] = $_POST['source_host'];
        $params['target_host'] = $_POST['target_host'];
        $params['filename'] = $_POST['filename'];
        $params['md5'] = $_POST['md5'];
        $params['sha512'] = $_POST['sha512'];		
	}
}	
else {
	if (!isset($_POST['md5'])) { $missing_param[] = "md5"; }
	if (!isset($_POST['sha512'])) { $missing_param[] = "sha512"; }
	if (!isset($_FILES['file'])) { $missing_param[] = "file"; }
	
	# Parameters missing - log and exit
	if (isset($missing_param) && is_array($missing_param)) {
		$log_tmp = sprintf('Submission parameters missing for "sample": %s',
			implode(" ", $missing_param)
		);
			
		$logger->log($log_tmp, PEAR_LOG_WARNING);
		echo "S_ERROR";
		exit;
	}
	
	# Parameters are fine - save in vars
	else {
        $params['sha512'] = $_POST['sha512'];
        $params['md5'] = $_POST['md5'];
        $params['file'] = $_FILES['file']['tmp_name'];		
	}
}

if ($funct == "instance") {
        $instance_idx = $db->nextID('instance');

        # Counter for total instances
		incrementStats("instance", "instance");
		
		# Counter for sensor transmissions
        incrementStats("sensor", $sensor['sensor_idx']);
            
        # Counter for source_host
		#incrementStats("source_host", $params['source_host']);
		
        # Counter for target_host
        #incrementStats("target_host", $params['target_host']);

		# Counter for a particular sample
		incrementStats("sample_sha512", $params['sha512']);

		# Counter for total samples
		incrementStats("sample", "sample");
	
        $query = sprintf($queries['insert_instance'],
                $instance_idx,
                $db->escape($params['email']),
                $db->escape($params['url']),
                $db->escape($params['trigger']),
                $db->escape($params['sha512']),
                $db->escape($params['filetype']),
                $db->escape($params['source_host']),
                $db->escape($params['target_host']),
                $db->escape($params['filename']),
                time(),
                $sensor['sensor_idx']);

        $logger->log("Inserting instance", PEAR_LOG_INFO);
        
        $logger->log("Inserting instance: $query", PEAR_LOG_DEBUG);

        $res = $db->query($query);

        if (PEAR::isError($res)) {
			$logger->log("DB error during insert_instance:", PEAR_LOG_EMERG); 
			$logger->log($res->getMessage(), PEAR_LOG_EMERG);
			echo "S_ERROR";
			exit;
        }

        # Further processing of instance data with modules
        foreach($mainModules as $module_name => $module_options) {
                
			if ($module_options['enable'] && ereg('submit', $module_options['scope'])
				&& $module_options['type'] == 'instance') {

				$logger->log("Retrieving instance_info from module $module_name", PEAR_LOG_INFO);
					
				require_once("modules/$module_name/$module_name.php");
                		
				$module = new $module_name();

                $instance_info_details = $module->process($params);
                	
				$query = sprintf($queries['insert_instance_info'],
					$db->nextID('instance_info'),
					$db->escape($module->info_type),
					$db->escape($module->info_name),
					$instance_idx,
					$db->escape($instance_info_details)
				);

				$logger->log("'Inserting instance_info_details", PEAR_LOG_INFO);
                         
				$logger->log("'Inserting instance_info_details: $query", PEAR_LOG_DEBUG);

				$db->query($query);
			}
		}	
 
		$query = sprintf($queries['update_sensor_last_submission'], time(), $sensor['sensor_idx']);
		$db->query($query);
		
		if (exists_sha512($params)) {
			echo "S_FILEKNOWN";
			exit;
		}
		else {
			echo "S_FILEREQUEST";
			exit;
		}
}

elseif ($funct == "sample") {
	# Counter for a particular sample
	#incrementStats("sample_sha512", $params['sha512']);
	
	# Counter for total samples
	#incrementStats("sample", "sample");
	
	if (exists_sha512($params)) {
		echo "S_OK";
		exit;	
	}

	else {
		$logger->log("Binary unknown - requesting", PEAR_LOG_DEBUG);
		
       	# Check MD5 sum of PHP temp file
        if ($params['md5'] == md5_file($params['file'])) {
        	
			# MD5 OK, copy file from PHP temp dir to our dir
			copy($params['file'], "$mainSamplePath/$params[sha512]") or die("Copy of temp file failed");
		
			# Create lockfile to prevent parallel processing of sample
			$lockfile = fopen("$mainSamplePath/$params[sha512].lck", "x");
			
            $query = sprintf($queries['insert_sample'],
				$db->escape($params['md5']),
				$db->escape($params['sha512']),
				time()
            );

            $db->query($query);
            
            # Sample inserted in DB, now process binary with modules
            foreach($mainModules as $module_name => $module_options) {
                if ($module_options['enable'] && ereg('submit', $module_options['scope'])
			 		&& $module_options['type'] == 'sample') {
                
                	require_once("modules/$module_name/$module_name.php");
                
                	$module = new $module_name();

                	$sample_info_details = $module->process($params);

                	$logger->log("module_param: $module_param sample_info_details: $sample_info_details", PEAR_LOG_DEBUG);
                	
                	if (!empty($sample_info_details)) {
						$query = sprintf($queries['insert_sample_info'],
							$db->nextID('sample_info'),
                            $db->escape($module->info_type),
                            $db->escape($module->info_name),
                            $db->escape($params['sha512']),
                            $db->escape($sample_info_details)
						);

						$logger->log("Inserting sample_info_details", PEAR_LOG_INFO);
                                
						$logger->log("Inserting sample_info_details: $query", PEAR_LOG_DEBUG);
						
						$db->query($query);
					}
					else {
						$logger->log("sample_info_details returned empty from $module_name", PEAR_LOG_DEBUG);
					}
                }
        	}
        	unlink("$mainSamplePath/$params[sha512].lck");
			echo "S_FILEOK";
        }
	    
        # File validation by md5 hash not OK
        else {
                $logger->log("Could not verify MD5 hash $params[md5] for $params[file]", PEAR_LOG_WARNING);
                unlink("$mainSamplePath/$params[sha512].lck");
                echo "S_ERROR";
                exit;
        }  
	}
}

else {
	$logger->log('Func is not set', PEAR_LOG_EMERG);
    echo "S_ERROR";
    exit;
}

?>
